"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Http2Client = void 0;
const http2_1 = require("http2");
const requestUtil_1 = require("./utils/requestUtil");
const responseUtil_1 = require("./utils/responseUtil");
const timeUtil_1 = require("./utils/timeUtil");
const { HTTP_STATUS_INTERNAL_SERVER_ERROR, HTTP2_HEADER_AUTHORITY, HTTP2_HEADER_CONTENT_TYPE, HTTP2_HEADER_METHOD, HTTP2_HEADER_PATH, HTTP2_HEADER_SCHEME, HTTP2_HEADER_STATUS, NGHTTP2_CANCEL, } = http2_1.constants;
const HTTP2_METHODS_WITH_PAYLOAD = ['POST', 'PUT'];
class Http2Client {
    constructor(url, options) {
        this.url = url;
        this.options = options;
        this.session = http2_1.connect(url, options);
    }
    close() {
        this.session.close();
    }
    async request(requestOptions, nativeOptions) {
        return new Promise((resolve, reject) => {
            if (this.session.closed || this.session.destroyed) {
                this._reconnect();
            }
            let status = -1;
            let responsePayloadType;
            const requestStartMilliSeconds = Date.now();
            const responseHeaders = {};
            const data = [];
            const requestBody = this._getRequestBody(requestOptions);
            const requestHeaders = {
                ...this._getRequestHeaders(requestOptions),
                ...(requestBody && { 'Content-Length': requestBody.length }),
                ...requestOptions.headers,
            };
            const req = this.session.request(requestHeaders, nativeOptions);
            req.setEncoding('utf8');
            req.setTimeout(requestOptions.inactiveTimeout || 2000, () => req.close(NGHTTP2_CANCEL));
            if (requestBody) {
                req.write(requestBody);
            }
            req.on('aborted', () => {
                console.log('stream aborted');
            });
            req.on('close', () => {
                resolve({
                    body: this._getResponseBody(data, responsePayloadType),
                    headers: responseHeaders,
                    statusCode: this._getResponseStatus(status),
                });
            });
            req.on('data', (chunk) => {
                if (requestOptions.activeTimeout &&
                    timeUtil_1.timestampIsInPast(requestStartMilliSeconds + requestOptions.activeTimeout)) {
                    req.close();
                }
                data.push(chunk);
            });
            req.on('error', (err) => {
                console.log('error');
                console.log({
                    name: err.name,
                    message: err.message,
                });
                reject(err);
            });
            req.on('frameError', (errorType, code, id) => {
                console.log('frame error');
                console.log({
                    errorType,
                    code,
                    id,
                });
                reject({ errorType, code, id });
            });
            req.on('response', (headers) => {
                if (headers[':status']) {
                    status = headers[':status'];
                }
                for (const name in headers) {
                    if (headers[name] && name !== HTTP2_HEADER_STATUS) {
                        const header = headers[name]?.toString();
                        if (header) {
                            responseHeaders[name] = header;
                            if (name === HTTP2_HEADER_CONTENT_TYPE) {
                                responsePayloadType = responseUtil_1.getResponsePayloadType(header);
                            }
                        }
                    }
                }
            });
            req.on('timeout', () => {
                console.log('stream timed out');
            });
        });
    }
    _getRequestBody(requestOptions) {
        if (requestOptions.method && HTTP2_METHODS_WITH_PAYLOAD.includes(requestOptions.method)) {
            const requestBody = requestOptions.body;
            if (requestBody) {
                switch (requestBody.type) {
                    case "BUFFER" /* BUFFER */:
                        return requestBody.data;
                    case "JSON" /* JSON */:
                        return Buffer.from(JSON.stringify(requestBody.data));
                    case "STRING" /* STRING */:
                        return Buffer.from(requestBody.data);
                }
            }
        }
        return;
    }
    _getRequestHeaders(options) {
        if (options.method === 'OPTIONS') {
            return {
                ...(options.authority && {
                    [HTTP2_HEADER_AUTHORITY]: options.authority,
                }),
                [HTTP2_HEADER_METHOD]: 'OPTIONS',
                ...(options.scheme && {
                    [HTTP2_HEADER_SCHEME]: options.scheme,
                }),
                'Content-Type': requestUtil_1.getContentTypeHeader(options.body?.type),
            };
        }
        else {
            return {
                ...(options.authority && {
                    [HTTP2_HEADER_AUTHORITY]: options.authority,
                }),
                [HTTP2_HEADER_METHOD]: options.method || 'GET',
                [HTTP2_HEADER_PATH]: (options.path || '/') + requestUtil_1.convertQueryParamsToUrl(options.queryParams),
                [HTTP2_HEADER_SCHEME]: options.scheme || 'https',
                'Content-Type': requestUtil_1.getContentTypeHeader(options.body?.type),
            };
        }
    }
    _getResponseBody(data, responseType) {
        switch (responseType) {
            case "BUFFER" /* BUFFER */:
                return data.join().toString();
            case "JSON" /* JSON */:
                return data.length === 0 ? {} : JSON.parse(data.join());
            case "STRING" /* STRING */:
            default:
                return data.join();
        }
    }
    _getResponseStatus(status) {
        return status === -1 ? HTTP_STATUS_INTERNAL_SERVER_ERROR : status;
    }
    _reconnect() {
        this.session = http2_1.connect(this.url, this.options);
    }
}
exports.Http2Client = Http2Client;
//# sourceMappingURL=client.js.map